﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7.8
@EndOfDesignText@
'Version 0.9 - JCO
'This version (renamed to v0.9) should run with B4A < 7.00
'For use with B4A v7.0+, use version 1.0+ of this class
'Requires OkHttpUtils2, JSON, RuntimePermissions
'Target SDK version in manifest must be at least 23
' and the following should be added in the manifest editor:
'	AddManifestText(<uses-permission
'		android:name="android.permission.WRITE_EXTERNAL_STORAGE"
'		android:maxSdkVersion="18" />
'	)
'You'll need an API key for openweathermap and edit the value for weatherAPIKey accordingly
'Under some conditions, those keys are free. See https://openweathermap.org/api
Sub Class_Globals
	Public weatherAPIKey As String = "YourAPIKeyForOpenweathermap.org"
	Public weatherCityID As String
	Public weatherUnits As String = "metric"
	
	Private KeyList As List = Array ("main", "description", "icon", "temp", "pressure", "humidity", "temp_min", "temp_max", "visibility", "wind_speed", "wind_degrees", "cloud_cover", "sunrise", "sunset")
	Private UnitList As List = Array ("", "", "", "°c", " hPa", "%", "°c", "°c", " meters", " m/s", "°", "%", "", "")
	Private ValueList As List = Array ("", "", "", "", "", "", "", "", "", "", "", "", "", "")
	Private CaptionList As List = Array ("Main", "Description", "Icon", "Temperature", "Pressure", "Humidity", "Low", "High", "Visibility", "Wind speed", "Wind direction", "Cloud cover", "Sunrise", "Sunset")
	Private CaptionMap As Map
	Private ValueMap As Map
	Private UnitMap As Map
	Private m_RefreshInterval As Int
	Private tmrInterval As Timer
	Private iconName As String
	Private folderWeather As String
	Private imgTmp As ImageView
	
	Private callBackActivity As Object
	Private eventPrefix As String
End Sub

'TargetActivity is the activity that will receive the ready and failed events
'CityId is the ID of the city as set in http://bulk.openweathermap.org/sample/city.list.json.gz
'RefreshInterval is the number of minutes after which a new call to the server will be made. Keep it not too short
'If GetLatest is true, a call for the latest information will immediately be made, if false, you'll have to call DownloadWeather() to get it
Public Sub Initialize(TargetActivity As Object, EventName As String, CityID As String, GetLatest As Boolean, RefreshInterval As Int)
	Dim rp As RuntimePermissions
	
	folderWeather = rp.GetSafeDirDefaultExternal("")
	
	weatherCityID = CityID
	callBackActivity = TargetActivity
	eventPrefix = EventName
	m_RefreshInterval = RefreshInterval
	tmrInterval.Initialize("tmrInterval", 1000)
	tmrInterval.Enabled = False
	
	Log(folderWeather & "\weather.txt")
	If File.Exists(folderWeather, "weather.txt") Then
		ValueMap = File.ReadMap(folderWeather, "weather.txt")
		Log(ValueMap)
	Else
		ValueMap.Initialize
	End If
	
	CaptionMap.Initialize
	UnitMap.Initialize
	For i = 0 To KeyList.Size-1
		CaptionMap.Put(KeyList.Get(i), CaptionList.Get(i))
		UnitMap.Put(KeyList.Get(i), UnitList.Get(i))
		If ValueMap.GetDefault(KeyList.Get(i), "") = "" Then
			ValueMap.Put(KeyList.Get(i), ValueList.Get(i))
		End If
	Next
	
	If GetLatest Then
		DownloadWeather
	End If
	
End Sub

'Makes a new call for the latest info and then raises _Ready or _Failed events
Public Sub DownloadWeather()
	Dim j As HttpJob
	Dim url As String = "http://api.openweathermap.org/data/2.5/weather"
	
	j.Initialize("main", Me)
	j.Download2(url, Array As String ("id", weatherCityID, "units", weatherUnits, "appid", weatherAPIKey))
	
End Sub

'Loads the icon corresponding to the current weather conditions. The file is downloaded if necessary
Public Sub LoadCurrentIcon(img As ImageView)
	
	If File.Exists(folderWeather, iconName) = False And (iconName <> "") Then
		Dim url As String = "http://openweathermap.org/img/w/" & iconName
		imgTmp = img
		img.Visible = False
		
		Dim j As HttpJob
		j.Initialize("icon", Me)
		j.Download(url)
	Else
		img.Bitmap = LoadBitmap(folderWeather, iconName)
		img.Visible = True
	End If
	
End Sub

Private Sub JobDone(Job As HttpJob)
	
	If Job.Success = False Then
		Select Job.JobName
			Case "main"
				If SubExists(callBackActivity, eventPrefix & "_Failed") Then
					CallSubDelayed(callBackActivity, eventPrefix & "_Failed")
				End If
			Case "icon"
				Log("Couldn't get weather icon" & CRLF & "Check Internet connection")
		End Select
		Job.Release
		Return
	End If
	
	Select Job.JobName
		Case "main"
			ParseWeatherData(Job.GetString)
			If m_RefreshInterval > 0 Then
				tmrInterval.Interval = m_RefreshInterval * 60000
				tmrInterval.Enabled = True
			End If
		Case "icon"
			Dim instr As InputStream = Job.GetInputStream
			Dim b(instr.BytesAvailable) As Byte
			instr.ReadBytes(b, 0, b.Length)
			BytesToFile(folderWeather, iconName, b)
			LoadCurrentIcon(imgTmp)
	End Select
	
	Job.Release
	
End Sub

private Sub tmrInterval_Tick()
	tmrInterval.Enabled = False
	DownloadWeather
End Sub

'Possible keys: 
'"main", "description", "icon", "temp", "pressure", "humidity", "temp_min", "temp_max", "visibility", "wind_speed", "wind_degrees", "cloud_cover", "sunrise", "sunset"
Public Sub GetValue(key As String) As String
	Return(ValueMap.GetDefault(key, ""))
End Sub

'Possible keys: 
'"main", "description", "icon", "temp", "pressure", "humidity", "temp_min", "temp_max", "visibility", "wind_speed", "wind_degrees", "cloud_cover", "sunrise", "sunset"
Public Sub GetCaption(key As String) As String
	Return(CaptionMap.GetDefault(key, ""))
End Sub

'Possible keys: 
'"main", "description", "icon", "temp", "pressure", "humidity", "temp_min", "temp_max", "visibility", "wind_speed", "wind_degrees", "cloud_cover", "sunrise", "sunset"
Public Sub GetUnit(key As String) As String
	Return(UnitMap.GetDefault(key, ""))
End Sub

Private Sub ParseWeatherData(jsonData As String)
	Dim js As JSONParser
	Dim root As Map
	Dim lst As List
	Dim map As Map
	Dim desc As String
	
	map.Initialize
	
	Log(jsonData)
	js.Initialize(jsonData)
	root.Initialize
	root = js.NextObject
	ValueMap.Put("visibility", NumberFormat2(root.Get("visibility"), 1, 0, 0, True))
	
	lst = root.Get("weather")
	map = lst.Get(0)
	ValueMap.Put("main", map.Get("main"))
	
	desc = map.Get("description")
	desc = desc.SubString2(0,1).ToUpperCase & desc.SubString(1)
	ValueMap.Put("description", desc)
	
	ValueMap.Put("icon", map.Get("icon"))
	iconName = ValueMap.Get("icon") & ".png"
	
	map = root.Get("main")
	ValueMap.Put("temp", NumberFormat2(map.Get("temp"), 1, 0, 0, False))
	ValueMap.Put("pressure", map.Get("pressure"))
	ValueMap.Put("humidity", map.Get("humidity"))
	ValueMap.Put("temp_min", map.Get("temp_min"))
	ValueMap.Put("temp_max", map.Get("temp_max"))
	
	map = root.Get("wind")
	ValueMap.Put("wind_speed", map.Get("speed"))
	ValueMap.Put("wind_degrees", map.Get("deg"))
	ValueMap.Put("cardinal", WindDegreesToCardinal(map.Get("deg")))
	
	map = root.Get("clouds")
	ValueMap.Put("cloud_cover", map.Get("all"))
	
	map = root.Get("sys")
	Dim tmp As String = DateTime.TimeFormat
	DateTime.dateFormat = "HH:mm"
	ValueMap.Put("sunrise", DateTime.Date((map.Get("sunrise"))*1000))
	ValueMap.Put("sunset", DateTime.Date((map.Get("sunset"))*1000))
	DateTime.dateFormat = tmp
	
	File.WriteMap(folderWeather, "weather.txt", ValueMap)
	
	If SubExists(callBackActivity, eventPrefix & "_Ready") Then
		CallSubDelayed(callBackActivity, eventPrefix & "_Ready")
	End If
	
End Sub

Private Sub WindDegreesToCardinal(deg As String) As String
	
	If IsNumber(deg) = False Then Return ""
	
	If deg < 11.25 Then Return "N"
	If deg < 33.75 Then Return "NNE"
	If deg < 56.25 Then Return "NE"
	If deg < 78.75 Then Return "ENE"
	If deg < 101.25 Then Return "E"
	If deg < 123.75 Then Return "ESE"
	If deg < 146.25 Then Return "SE"
	If deg < 168.75 Then Return "SSE"
	If deg < 191.25 Then Return "S"
	If deg < 213.75 Then Return "SSW"
	If deg < 236.25 Then Return "SW"
	If deg < 258.75 Then Return "WSW"
	If deg < 281.25 Then Return "W"
	If deg < 303.75 Then Return "WNW"
	If deg < 326.25 Then Return "NW"
	If deg < 348.75 Then Return "NNW"
	If deg < 360 Then Return "N"
	Return ""
	
End Sub

Private Sub BytesToFile (Dir As String, FileName As String, Data() As Byte)
	Dim out As OutputStream = File.OpenOutput(Dir, FileName, False)
	out.WriteBytes(Data, 0, Data.Length)
	out.Close
End Sub
